@@ -7,5 +7,4 @@ import gym_pool.envs.table_sprites
 import gym_pool.envs.collisions
 import gym_pool.envs.gamestate
 from gym_pool.envs.pool_env import PoolEnv
-#from gym_soccer.envs.soccer_empty_goal import SoccerEmptyGoalEnv
-#from gym_soccer.envs.soccer_against_keeper import SoccerAgainstKeeperEnv
+from gym_pool.envs.pool_env_3 import PoolEnv3
