@@ -3,21 +3,22 @@ import math
 import random
 from pprint import pprint
 from enum import Enum
+import os
 
 import numpy as np
 import pygame
 import zope.event
 
-import gym_pool.envs.ball as ball
-import gym_pool.envs.config as config
-import gym_pool.envs.cue as cue
-import gym_pool.envs.cue_visible as cue_visible
-import gym_pool.envs.event as event
-import gym_pool.envs.graphics as graphics
-import gym_pool.envs.table_sprites as table_sprites
-from gym_pool.envs.ball import BallType
-from gym_pool.envs.collisions import check_if_ball_touches_balls
-import gym_pool.envs.collisions as collisions
+from . import ball
+from . import config
+from . import cue
+from . import cue_visible
+from . import event
+from . import graphics
+from . import table_sprites
+from .ball import BallType
+from .collisions import check_if_ball_touches_balls
+from . import collisions
 
 
 class Player(Enum):
@@ -31,15 +32,20 @@ class InterState(object):
     pass
 
 class GameState:
-    def __init__(self, ball_num, visualize=False):
+    def __init__(self, ball_num, visualize=False, singlePlayer = False):
         self.visualize = visualize
+        if not visualize:
+            os.environ["SDL_VIDEODRIVER"] = "dummy"
+        self.singlePlayer =singlePlayer
         pygame.init()
         pygame.display.set_caption(config.window_caption)
         event.set_allowed_events()
         zope.event.subscribers.append(self.game_event_handler)
-        self.canvas = graphics.Canvas()
+        self.canvas = graphics.Canvas(visualize)
         self.ball_num = ball_num
         self.collision_count = 0
+        self.white_in=0
+        self.eight_in=0
         self.fps_clock = pygame.time.Clock()
         self.start_pool(ball_num)
         events = event.events()
@@ -47,6 +53,12 @@ class GameState:
 
     def fps(self):
         return self.fps_clock.get_fps()
+    #NEW METHOD
+    def get_is_game_over(self):
+        #Is only the white ball left? Then game over
+        if len(self.balls.sprites()) == 1 and self.balls.sprites()[0].number == 0:
+            return True
+        return False
 
     def mark_one_frame(self):
         self.fps_clock.tick(2000)
@@ -73,7 +85,6 @@ class GameState:
                 self.all_sprites.remove(event.data)
             self.potted.append(event.data.number)
         elif event.type == "COLLISION":
-            self.collision_count += 1
             if not self.white_ball_1st_hit_is_set:
                 self.first_collision(event.data)
 
@@ -104,15 +115,17 @@ class GameState:
         self.reset_state()
         self.generate_table()
         self.set_pool_balls(ball_num)
-        if not self.visualize:
-            self.cue = cue.Cue(self.white_ball)
-        else:
-            self.cue = cue_visible.Cue(self.white_ball)
-            #self.cue = cue.Cue(self.white_ball)
+        self.cue = cue_visible.Cue(self.white_ball)
+        #if not self.visualize:
+        #    self.cue = cue.Cue(self.white_ball)
+        #else:
+            #self.cue = cue_visible.Cue(self.white_ball)
+        #    self.cue = cue.Cue(self.white_ball)
         self.all_sprites.add(self.cue)
 
     def reset_state(self):
         # game state variables
+        self.first = True
         self.current_player = Player.Player1
         self.turn_ended = True
         self.white_ball_1st_hit_is_set = False
@@ -163,14 +176,16 @@ class GameState:
         # holes_x and holes_y holds the possible xs and ys of the table holes
         # with a position ID in the second tuple field
         # so the top left hole has id 1,1
-        holes_x = [(config.table_margin, 1), (config.resolution[0] /
+        self.holes_x = [(config.table_margin, 1), (config.resolution[0] /
                                               2, 2), (config.resolution[0] - config.table_margin, 3)]
-        holes_y = [(config.table_margin, 1),
+        self.holes_y = [(config.table_margin, 1),
                    (config.resolution[1] - config.table_margin, 2)]
+        print('holes x', self.holes_x)
+        print('holes y', self.holes_y)
         # next three lines are a hack to make and arrange the hole coordinates
         # in the correct sequence
         all_hole_positions = np.array(
-            list(itertools.product(holes_y, holes_x)))
+            list(itertools.product(self.holes_y, self.holes_x)))
         all_hole_positions = np.fliplr(all_hole_positions)
         all_hole_positions = np.vstack(
             (all_hole_positions[:3], np.flipud(all_hole_positions[3:])))
@@ -243,17 +258,26 @@ class GameState:
             self.can_move_white_ball = True
 
     def check_potted(self):
-        self.can_move_white_ball = False  # if white ball is potted, it will be created again and placed in the middle
+        if not self.first:
+            self.can_move_white_ball = False  # if white ball is potted, it will be created again and placed in the middle
         if 0 in self.potted:
+            self.white_in+=1
+            self.potted.remove(0)
             self.create_white_ball()
             self.cue.target_ball = self.white_ball
-            self.potted.remove(0)
-            self.turn_over(True)
-        #if 8 in self.potted:
-        #    if self.potting_8ball[self.current_player]:
-        #        self.game_over(self.current_player == Player.Player1)
-        #    else:
-        #        self.game_over(self.current_player != Player.Player1)
+            if not self.singlePlayer:
+                self.turn_over(True)
+            #else:
+                #print("Times white ball potted:",self.white_in)
+            return True
+        if 8 in self.potted:
+            self.eight_in+=1
+            if self.potting_8ball[self.current_player]:
+                self.game_over(self.current_player == Player.Player1)
+            else:
+                self.game_over(self.current_player != Player.Player1)
+            return True
+        return False
 
     def check_remaining(self):
         # a check if all striped or solid balls were potted
@@ -269,6 +293,16 @@ class GameState:
         self.potting_8ball = {Player.Player1: not ball_type_remaining[self.ball_assignment[Player.Player1]],
                               Player.Player2: not ball_type_remaining[self.ball_assignment[Player.Player2]]}
 
+    def check_remaining_single(self):
+        # a check if all solid balls were potted
+        solids_remaining = False
+        for remaining_ball in self.balls:
+            if remaining_ball.number != 0 and remaining_ball.number != 8:
+                solids_remaining = solids_remaining or not remaining_ball.ball_type == BallType.Striped
+
+        # decides if palyer should be potting 8ball
+        self.potting_8ball = {Player.Player1: not solids_remaining, Player.Player2: False}
+
     def first_collision(self, ball_combination):
         self.white_ball_1st_hit_is_set = True
         self.white_ball_1st_hit_8ball = ball_combination[0].number == 8 or ball_combination[1].number == 8
@@ -278,11 +312,20 @@ class GameState:
             self.white_ball_1st_hit_type = ball_combination[0].ball_type
 
     def check_pool_rules(self):
-        if self.ball_assignment is not None:
-            self.check_remaining()
-        self.check_potted()
+        if self.first:
+            self.first = False
+            self.check_potted()
+            self.on_next_hit()
+            return
+        if not self.singlePlayer:
+            self.potted_ball_rules()
+            if self.ball_assignment is not None:
+                self.check_remaining()
+        if self.singlePlayer:
+            self.check_remaining_single()
+
         self.first_hit_rule()
-        self.potted_ball_rules()
+        self.check_potted() # checks whether white ball was potted
         self.on_next_hit()
 
     def on_next_hit(self):
@@ -291,7 +334,7 @@ class GameState:
         self.potted = []
 
     def potted_ball_rules(self):
-        if len(self.potted) > 0:
+        if len(self.potted) > 0 and not self.singlePlayer:
             # if it wasnt decided which player goes for which type of balls
             # and the player potted the balls exclusively of one color (excluting white balls)
             # then it is decided based on which players turn it is right now and which type
@@ -319,8 +362,12 @@ class GameState:
         # for example if the current player hits a striped ball with the whit ball
         # but he should be potting solid balls, it is next players turn and he can move the white ball
         if not self.white_ball_1st_hit_is_set:
-            self.turn_over(True)
-        elif self.ball_assignment is not None:
+            self.collision_count += 1
+            if not self.singlePlayer:
+                self.turn_over(True)
+            #else:
+                #print("Times missed ball:",self.collision_count)
+        elif self.ball_assignment is not None and not self.singlePlayer:
             if not self.white_ball_1st_hit_8ball and self.ball_assignment[
                 self.current_player] != self.white_ball_1st_hit_type:
                 self.turn_over(True)
@@ -328,6 +375,12 @@ class GameState:
             # and if not he gets penalised
             elif self.white_ball_1st_hit_8ball:
                 self.turn_over(not self.potting_8ball[self.current_player])
+        #elif self.singlePlayer:
+            # checks if the 8ball was the first ball hit, and if so checks if the player needs to pot the 8ball
+            # and if not he gets penalised
+            #if self.white_ball_1st_hit_8ball and not self.potting_8ball[self.current_player]:
+                #print('Times 8ball potted:',self.eight_in)
+
     def return_game_state(self):
         state = InterState()
         state.balls=[]
@@ -351,9 +404,62 @@ class GameState:
                 state.append(currentBall)
         return state
 
+    def return_ball_state_dict(self):
+        state = {}
+        for i in range(0, len(self.balls.sprites())):
+            currentBall = (self.balls.sprites()[i].ball.pos[0], self.balls.sprites()[i].ball.pos[1])
+            state[self.balls.sprites()[i].number] = [currentBall]
+        try:
+            white_ball = state[0][0]
+        except:
+            self.check_potted()
+            self.balls.add(self.white_ball)
+            state = {}
+            for i in range(0, len(self.balls.sprites())):
+                currentBall = (self.balls.sprites()[i].ball.pos[0], self.balls.sprites()[i].ball.pos[1])
+                state[self.balls.sprites()[i].number] = [currentBall]
+            white_ball = state[0][0]
+        for key in state.keys():
+            if key == 0:
+                state[key].extend([0, 0])
+            else:
+                state[key].append(self.euclidean_distance(white_ball, state[key][0]))
+                state[key].append(self.angle(white_ball, state[key][0]))
+            # add holes distances
+            for y in self.holes_y:
+                for x in self.holes_x:
+                    state[key].append(self.euclidean_distance((x[0], y[0]), state[key][0]))
+                    state[key].append(self.angle((x[0], y[0]), state[key][0]))
+                break
+        # for k,b in pos_dict.items():
+        #   print(k,b)
+        state = list(state.items())
+        full_state = []
+        for i in range(1,self.ball_num):
+            try:
+                self.balls.sprites()[i].number
+                #full_state += [state[i][1][0][0], state[i][1][0][1]] + state[i][1][1:]
+                full_state += state[i][1][1:]
+            except:
+                full_state += [0,0,0,0,0,0,0,0,]
+        full_state = np.array(full_state).flatten()
+        return full_state
+
+    def euclidean_distance(self, point1, point2):
+        return np.sqrt(np.square((point2[0] - point1[0])) + np.square((point2[1]-point1[1])))
+
+    def angle(self, point1, point2):
+        unit_vector_1 = point1/ np.linalg.norm(point1)
+        unit_vector_2 = point2 / np.linalg.norm(point2)
+        dot_product = np.dot(unit_vector_1, unit_vector_2)
+        angle = np.arccos(dot_product)
+        return angle
+
     def step(self, game, angle, force):
-        self.collision_count = 0
         original_pos = self.return_ball_state()
+        self.collision_count_old = self.collision_count
+        self.white_in_old = self.white_in
+        self.eight_in_old = self.eight_in
         ang_in_max = 0
         ang_in_min = 1
 
@@ -393,11 +499,13 @@ class GameState:
         #print('Num steps was ' + str(num_steps))
 
         new_pos = self.return_ball_state()
+        pos_dict = self.return_ball_state_dict()
+
         balls_in = len(original_pos) - len(new_pos)
         done = 1 if len(new_pos) == 1 else 0
 
         x = len(new_pos)
         for i in range(x, self.ball_num):
             new_pos.append((0, 0))
-        collision_count = self.collision_count
-        return (new_pos, balls_in, collision_count, done)
+        #distance, position and angle should be def in there
+        return (pos_dict, balls_in, self.collision_count-self.collision_count_old, self.white_in-self.white_in_old, self.eight_in-self.eight_in_old, done)
