@@ -1,22 +1,17 @@
 import numpy as np
 import functools
 import operator
-#import gym_pool.envs.utils as utils
+import gym_pool.envs.utils as utils
 import gym_pool.envs.gamestate as gamestate
 import gym_pool.envs.collisions as collisons
 import gym_pool.envs.event as event
 import gym
 from gym import error, spaces
-from gym import utils
+from gym import utils as gym_utils
 from gym.utils import seeding
 import logging
 logger = logging.getLogger(__name__)
 
-
-class Utils:
-    def prod(l):
-        return functools.reduce(operator.mul, l, 1)
-
 class ActionSpace:
     def __init__(self, ranges):
         self.ranges = ranges
@@ -126,10 +121,10 @@ class StateSpace:
 
             return encoded_state
 
-class PoolEnv(gym.Env, utils.EzPickle):
+class PoolEnv(gym.Env, gym_utils.EzPickle):
     metadata = {'render.modes': ['human']}
 
-    def __init__(self, num_balls=2, visualize=False):
+    def __init__(self, num_balls=2, visualize=True):
         self.num_balls = num_balls
         self.visualize = visualize
 
